
import os.path
import os
import logging
import socket
import subprocess
import status
import simplejson
import shlex
import sys
import struct
import array


from turbogears import controllers, expose, error_handler, validate
from turbogears import flash, redirect, url

from netborderPyInfra import PyGlobalConfig as globalConfig
from netborderPyInfra import *
from paraxip.gateway_gui import extjs
from paraxip.gateway_gui_widgets.wizardform import WizardForm

log = logging.getLogger("pstnconfig")

class WizardController(controllers.Controller):
  def __init__(self, in_strProjectPath):
    strSrcFile = os.path.join(in_strProjectPath, "pstnconfig/static/javascript/"
                              "widgets_src/wizard_form_src.js")

    # Load the default ports values from the configuration
    defaultUDPPort = \
      globalConfig.getParameter("netborder.oamWebServer.defaultUDPListenPort")
    defaultTCPPort = \
      globalConfig.getParameter("netborder.oamWebServer.defaultTCPListenPort")
    defaultTLSPort = \
      globalConfig.getParameter("netborder.oamWebServer.defaultTLSListenPort")
    defaultUDPTCPSendPort = \
      globalConfig.getParameter("netborder.oamWebServer.defaultUDPTCPSendPort")
    defaultTLSSendPort = \
      globalConfig.getParameter("netborder.oamWebServer.defaultTLSSendPort")
    defaultPrimarySipServer = \
      globalConfig.getParameter("netborder.oamWebServer.defaultPrimarySipServer")

    startGwTimeout = \
      globalConfig.getParameter("netborder.oamWebServer.startGwTimeoutMs")

    self.wizard_form = WizardForm('wizardForm',
                                  title="Wizard",
                                  src_file=strSrcFile,
                                  defaultUDPPort=defaultUDPPort,
                                  defaultTCPPort=defaultTCPPort,
                                  defaultTLSPort=defaultTLSPort,
                                  defaultUDPTCPSendPort=defaultUDPTCPSendPort,
                                  defaultTLSSendPort=defaultTLSSendPort,
                                  defaultPrimarySipServer=defaultPrimarySipServer,
                                  startGwTimeout=startGwTimeout)

    # Try to get the value of the parameter postQuickSetupCmd to know if we
    # have to execute a script after the Quick Setup
    self.postQuickSetupCmd = \
      globalConfig.getParameter("netborder.oamWebServer.postQuickSetupCmd")

    # Installation Directory
    self.root_path = \
      globalConfig.getParameter("netborder.Installation.Directory")

    # Full path to the parameter DataBase file
    self.strParamDBFile = \
      os.path.join(self.root_path, "config/netborder-gateway-paramDefs.xml")

    # Load the netborder-gateway-paramDefs.xml file
    self.paramDB = ParameterInfoDB.loadFromXmlFile( self.strParamDBFile );
    if self.paramDB is None :
        log.error("failed to load parameter DB from file: " +
                  self.strParamDBFile );
        sys.exit(1);

    # Full path to properties file of the gateway
    self.strGwPropFile = \
      os.path.join(self.root_path, "config/gw.properties")

    # Vector that contains the properties file to load
    self.vInputFilePropertiesFiles = StringVector()
    self.vInputFilePropertiesFiles.extend( [self.strGwPropFile] )


  def getWizardForm (self):
    return self.wizard_form

  @expose(template='pstnconfig.templates.wizard')
  def lightbox_quick_setup(self):
    if globalConfig.getParameter("netborder.oamWebServer.offerVirtualBoards"):
      offerVirtualBoards = "true"
    else:
      offerVirtualBoards = "false"

    if globalConfig.getParameter("netborder.oamWebServer.isGatewayInCLI"):
      isGatewayInCLI = "true"
    else:
      isGatewayInCLI = "false"

    if globalConfig.getParameter("netborder.oamWebServer.enableHardwareDetection"):
      autoDetectHardware = "true"
    else:
      autoDetectHardware = "false"

    defaultUDPPort = \
      globalConfig.getParameter("netborder.oamWebServer.defaultUDPListenPort")
    defaultTCPPort = \
      globalConfig.getParameter("netborder.oamWebServer.defaultTCPListenPort")
    defaultTLSPort = \
      globalConfig.getParameter("netborder.oamWebServer.defaultTLSListenPort")
    defaultUDPTCPSendPort = \
      globalConfig.getParameter("netborder.oamWebServer.defaultUDPTCPSendPort")
    defaultTLSSendPort = \
      globalConfig.getParameter("netborder.oamWebServer.defaultTLSSendPort")
    defaultPrimarySipServer = \
      globalConfig.getParameter("netborder.oamWebServer.defaultPrimarySipServer")

    startGwTimeout = \
      globalConfig.getParameter("netborder.oamWebServer.startGwTimeoutMs")

    return \
      {
        "footerText":""
        ,"offerVirtualBoards":offerVirtualBoards
        ,"isGatewayInCLI":isGatewayInCLI
        ,"autoDetectHardware":autoDetectHardware
        ,"defaultUDPPort":defaultUDPPort
        ,"defaultTCPPort":defaultTCPPort
        ,"defaultTLSPort":defaultTLSPort
        ,"defaultUDPTCPSendPort":defaultUDPTCPSendPort
        ,"defaultTLSSendPort":defaultTLSSendPort
        ,"defaultPrimarySipServer":defaultPrimarySipServer
        ,"startGwTimeout":startGwTimeout
      }

  @expose()
  def index(self, tg_errors=None):
    """Show the pstnconfig form."""

    if tg_errors:
      flash('There was a problem with the form!')
      #isdnForm = isdn_wizard_form
    return {}


    ''' -- 'save' controller method --
     Configuration obtained from wizard sent under 'boardConfig' and
       'physicalConfig' keys in JSON format

     Examples:
    'boardConfigs': u'[{"board_id":1,
                       "model":"A108",
                       "num_interfaces":4,
                       "wanpipe":1}]'
    'physicalConfig': u'{"interface_type":"T1",
                         "framing_T1":"ESF",
                         "encoding_T1":"B8ZS",
                         "isdn_sw_T1":"5ESS",
                         "sip_url":"sip:localhost:5060",
                         "outbound_mask":"([\+]?[-\*\#\d]+)",
                         "outbound_prefix":"",
                         "country_code":"1",
                         "udpPort":"5066",
                         "outTransportProtocol":"udp"}',
    '''
  @expose() # Invoked by Ext JS Form "submit": JSON response expected
  @error_handler(index)
  def save(self, **data):
    """Handle submission from the comment form and save the comment."""

    log.info("Data values from POST request: %s" % data)

    import pstnconfig.model as model
    log.info("model: %s" % str(model.createDefaultSangomaPstnConfig))
    model.currentConfig = model.createDefaultSangomaPstnConfig()

    digitalBoards_PRI = []
    digitalBoards_BRI = []
    analogBoards = []
    virtualDigitalBoards = []
    phoneNumbersArray = []

    bHasFxsInterfaces = False
    bHasFxoInterfaces = False

    # 'boardConfigs' key value is a JSON list
    for boardConfig in simplejson.loads(data['boardConfigs']):
      # TODO make sure that board IDs will always be in order
      # TODO using the same key names for the js and the Python would make
      #   it easier
      log.debug("Board config: %s" % boardConfig)
      boardModel = boardConfig['model']
      # Make sure is a known model
      assert model.boardsModelInfo.has_key(boardModel)
      boardModelInfo = model.boardsModelInfo[boardModel]

      if model.hardwareDetectionEnabled:
        if not boardConfig['active']:
          # board was not selected. do not add it to the config
          continue

      if boardModelInfo['type'] == 'digital':
        if model.hardwareDetectionEnabled:
          for i in simplejson.loads(data['interfacesData']):
            if i['boardID'] == boardConfig['id']:
              ifData = i['interfacesData']
      
          digitalBoard = model.addSangomaDigitalBoardHwMode(
            model.currentConfig,
            boardModel=boardConfig['model'],
            boardID=boardConfig['id'],
            displayName=boardConfig['displayName'],
            pciBus=boardConfig['pciBus'],
            pciSlot=boardConfig['pciSlot'],
            interfacesData=ifData
          )
        else:
          
          if boardModelInfo['digitalType'] == 'BRI':
                   
            digitalBoard = model.addSangomaDigitalBoard(
              model.currentConfig,
              boardModel=boardModel,
              boardID=boardConfig['board_id'],
              starting_wanpipe=boardConfig['wanpipe'],
              briModType=boardConfig['briModType'])
          else:
            digitalBoard = model.addSangomaDigitalBoard(
              model.currentConfig,
              boardModel=boardModel,
              boardID=boardConfig['board_id'],
              starting_wanpipe=boardConfig['wanpipe']
            )
          
        if boardModelInfo['digitalType'] == 'PRI':
          digitalBoards_PRI.append(digitalBoard)
        elif boardModelInfo['digitalType'] == 'BRI':
          digitalBoards_BRI.append(digitalBoard)

      elif boardModelInfo['type'] == 'analog':
        if model.hardwareDetectionEnabled:
          for i in simplejson.loads(data['interfacesData']):
            if i['boardID'] == boardConfig['id']:
              ifData = i['interfacesData']

          analogBoards.append(
            model.addSangomaAnalogBoardHwMode(
              model.currentConfig,
              boardModel=boardConfig['model'],
              boardID=boardConfig['id'],
              displayName=boardConfig['displayName'],
              pciBus=boardConfig['pciBus'],
              pciSlot=boardConfig['pciSlot'],
              wanpipe=boardConfig['wanpipe'],
              interfacesData=ifData
            )
          )

        else:
          analogBoards.append(
            model.addSangomaAnalogBoard(
              model.currentConfig,
              boardModel=boardModel,
              boardID=boardConfig['board_id'],
              wanpipe=boardConfig['wanpipe'],
              numFxoInterfaces=boardConfig['numFxoInterfaces'],
              numFxsInterfaces=boardConfig['numFxsInterfaces'],
              phoneNumbers=boardConfig['phoneNumbersArray'],
              callerNames=boardConfig['callerNamesArray']
            )
          )

        if model.boardHasFxsInterfaces(analogBoards[-1]):
          bHasFxsInterfaces = True
        
        if model.boardHasFxoInterfaces(analogBoards[-1]):
          bHasFxoInterfaces = True

      elif boardModelInfo['type'] == 'virtual_digital':
        virtualDigitalBoards.append(
          model.addSangomaVirtualDigitalBoard(
            model.currentConfig,
            boardID=boardConfig['board_id'],
            numInterfaces=boardConfig['num_interfaces']
          )
        )
      else:
        assert 'Unsupported board model type'

    # Add physical config to the model
    physicalConfig = simplejson.loads(data['physicalConfig'])
    interfaceType = physicalConfig['interface_type']

    if len(digitalBoards_PRI) > 0:

      if interfaceType == "T1" or interfaceType == "J1":
        #Create the config parameters dictionary
        ConfigParams = {}
        ConfigParams['lineEncoding'] = \
          physicalConfig['encoding_%s' % interfaceType]
        ConfigParams['clocking'] = 'TERMINAL'
        ConfigParams['loopbackMode'] = 'DISABLED'
        ConfigParams['framing'] = \
          physicalConfig['framing_%s' % interfaceType]
        ConfigParams['LBO'] = '0dB'

        #Create the audio parameters dictionary
        AudioParams = model.defaultAudioParams.copy()
        AudioParams['toneSetName'] = physicalConfig['country']
        
        if interfaceType == "T1":
          digitalConfig = model.addT1SangomaDigitalConfig(model.currentConfig,
              'P%s-digital_%s' % (model.getNextPhysConfID(model.currentConfig), 
                                  interfaceType),ConfigParams,AudioParams)
        elif interfaceType == "J1":
          digitalConfig = model.addJ1SangomaDigitalConfig(model.currentConfig,
              'P%s-digital_%s' % (model.getNextPhysConfID(model.currentConfig), 
                                  interfaceType),ConfigParams,AudioParams)

        for board in digitalBoards_PRI:
          model.applyConfigToBoardInterfaces(digitalConfig, board)

        # Indicate FAS config applies to all channels of all digital boards
        BChannelRange_PRI = model.BChannelRange.factory(firstChannel=1,
            lastChannel=23, channelGroupID='default')
        
        #Create FAS config
        callControlConfig = model.addFasConfig(model.currentConfig,
          displayName='FAS1-%s' % interfaceType,
          termination='TERMINAL',
          switchVariant=physicalConfig['isdn_sw_%s' % interfaceType],
          BChannelNegotiation='EXCLUSIVE',
          initiateRestartProcedure='YES',
          inbandProgressTonesGeneration='AS-NEEDED',
          inbandProgressTonesIndicator='PROGRESS',
          callerNameLocation='IN-SETUP-MSG-USING-DISPLAY-IE',
          waitFacilityDelayMs=100)
        

      elif interfaceType == "E1":
        #Create the config parameters dictionary
        E1ConfigParams = {}
        E1ConfigParams['lineEncoding'] = \
          physicalConfig['encoding_%s' % interfaceType]
        E1ConfigParams['E1Signaling'] = \
          physicalConfig['signaling_%s' % interfaceType]
        E1ConfigParams['clocking'] = 'TERMINAL'
        E1ConfigParams['loopbackMode'] = 'DISABLED'
        E1ConfigParams['framing'] = \
          physicalConfig['framing_%s' % interfaceType]
        E1ConfigParams['E1Termination'] = '120ohm'

        #Create the audio parameters dictionary
        E1AudioParams = model.defaultAudioParams.copy()
        E1AudioParams['toneSetName'] = physicalConfig['country']
        E1AudioParams['defaultPcmLaw'] = 'PCMA'

        digitalConfig = model.addE1SangomaDigitalConfig(model.currentConfig,
            'P%s-digital_%s' % (model.getNextPhysConfID(model.currentConfig),
            interfaceType), E1ConfigParams, E1AudioParams)

        for board in digitalBoards_PRI:
          model.applyConfigToBoardInterfaces(digitalConfig, board)

        callControlConfig = None
        if E1ConfigParams['E1Signaling'] == "CCS":
          callControlConfig = model.addFasConfig(model.currentConfig,
            displayName='FAS1-%s' % interfaceType,
            termination='TERMINAL',
            switchVariant="ETSI",
            BChannelNegotiation='EXCLUSIVE',
            initiateRestartProcedure='YES',
            inbandProgressTonesGeneration='AS-NEEDED',
            inbandProgressTonesIndicator='PROGRESS',
            callerNameLocation='IN-SETUP-MSG-USING-DISPLAY-IE',
            waitFacilityDelayMs=100)
        elif E1ConfigParams['E1Signaling'] == "CAS":
          
          getANIFirstDefaultValue = "TRUE"
          if physicalConfig["countryR2"] == "Mexico":
            getANIFirstDefaultValue = "FALSE"

          callControlConfig = model.addR2Config(
            model.currentConfig
            ,displayName='new_R2_%s' % model.getNextCASConfID(model.currentConfig)
            ,switchVariant=physicalConfig["countryR2"]
            ,get_ani_first=getANIFirstDefaultValue
            ,max_ani=10
            ,max_dnis=10
            ,category="National Subscriber"
            ,skip_category="FALSE"
            ,allow_collect_calls="FALSE"
            ,charge_calls="FALSE"
            ,immediate_accept="FALSE"
            ,double_answer="FALSE"
            ,forced_release="FALSE"
            ,mfback_timeout=2000
            ,call_files="FALSE")  
          
        # Indicate FAS config applies to all channels of all digital boards
        BChannelRange_PRI = model.BChannelRange.factory(firstChannel=1,
            lastChannel=30, channelGroupID='default')
    
    
      
  
      for board in digitalBoards_PRI:
        model.applyConfigToBoardInterfaces(callControlConfig, 
                                           board, 
                                           [BChannelRange_PRI])
      
    
    if len(digitalBoards_BRI) > 0:
      
      # Count the number of interfaces NETWORK and TERMINAL [Clocking]
      nbInterfaces_NT = 0 #nb interfaces with a NETWORK clocking
      nbInterfaces_TE = 0 #nb interfaces with a TERMINAL clocking
      for board in digitalBoards_BRI:
        for interface in board.get_interface():
          if interface.get_briModType() == "NETWORK":
            nbInterfaces_NT +=1
          elif interface.get_briModType() == "TERMINAL":
            nbInterfaces_TE +=1

       #Create the config for the two clocking type (NT and TE)
      ConfigParams = {}
      ConfigParams['BRISignaling'] = 'CCS'
      ConfigParams['loopbackMode'] = 'DISABLED'
      ConfigParams['LBO'] = '0dB'
      
      #Create the audio parameters dictionary
      AudioParams = model.defaultBriAudioParams.copy()
      AudioParams['toneSetName'] = physicalConfig['country']
      
      if(nbInterfaces_NT > 0):
        ConfigParams['clocking'] = 'NETWORK'
        digitalConfigNT = model.addBRISangomaDigitalConfig(model.currentConfig,
          'P%s-digital_%s_NT' % (model.getNextPhysConfID(model.currentConfig), 
                              'BRI'),ConfigParams,AudioParams)
        #Create and apply BRI config for NETWORK
        briConfigNT = model.addBRIConfig(model.currentConfig,
          displayName='BRI1_NT',
          termination='NETWORK',
          switchVariant="ETSI",
          BChannelNegotiation='EXCLUSIVE',
          initiateRestartProcedure='NO',
          inbandProgressTonesGeneration='AS-NEEDED',
          inbandProgressTonesIndicator='PROGRESS',
          callerNameLocation='IN-SETUP-MSG-USING-DISPLAY-IE',
          waitFacilityDelayMs=100,
          topology='MULTIPOINT',
          multipleSubscriberNumbers='',
          serviceType='VOICE',
          aocBehavior='NONE')
        
      if(nbInterfaces_TE > 0):
        ConfigParams['clocking'] = 'TERMINAL'
        digitalConfigTE = model.addBRISangomaDigitalConfig(model.currentConfig,
          'P%s-digital_%s_TE' % (model.getNextPhysConfID(model.currentConfig), 
                              'BRI'),ConfigParams,AudioParams)
        
        #Create and apply BRI config for TERMINAL
        briConfigTE = model.addBRIConfig(model.currentConfig,
          displayName='BRI1_TE',
          termination='TERMINAL',
          switchVariant="ETSI",
          BChannelNegotiation='EXCLUSIVE',
          initiateRestartProcedure='NO',
          inbandProgressTonesGeneration='AS-NEEDED',
          inbandProgressTonesIndicator='PROGRESS',
          callerNameLocation='IN-SETUP-MSG-USING-DISPLAY-IE',
          waitFacilityDelayMs=100,
          topology='MULTIPOINT',
          multipleSubscriberNumbers='',
          serviceType='VOICE',
          aocBehavior='NONE')
      
      #Apply the right config to each interfaces (physical and ISDN)
      BChannelRange_BRI = model.BChannelRange.factory(firstChannel=1,
          lastChannel=2, channelGroupID='default')
      
      for board in digitalBoards_BRI:
       for interface in board.get_interface():
         log.info("interface: " + str(interface))
         if interface.get_briModType() == "NETWORK":
           model.applyConfigToInterface(digitalConfigNT, board, interface)
           model.applyConfigToInterface(briConfigNT, board, interface, [BChannelRange_BRI])
         elif interface.get_briModType() == "TERMINAL":
           model.applyConfigToInterface(digitalConfigTE, board, interface)
           model.applyConfigToInterface(briConfigTE, board, interface, [BChannelRange_BRI])
           
    # Analog config
    if len(analogBoards):

      variant = physicalConfig['country']
      if variant == "TBR21":
        defaultPcmLaw = "PCMA"
      else:
        defaultPcmLaw = "PCMU"

      AudioParams = model.defaultAudioParams.copy()
      AudioParams['defaultPcmLaw'] = defaultPcmLaw
      AudioParams['toneSetName'] = physicalConfig['country']

      analogConfig = model.addSangomaAnalogConfig(model.currentConfig,
          'P%s-analog' % model.getNextPhysConfID(model.currentConfig),
          variant,
          "INTERNAL",
          AudioParams)

      # Analog physical config applies to each analog board
      for board in analogBoards:
        model.applyConfigToBoard(analogConfig, board)

      for board in analogBoards:
        result = self.addAnalogConfigs(
          model,
          board,
          phoneNumbersArray
        )
        if result == False:
          return(str(extjs.FormActionSubmitResponse(success=False)))


    # Virtual digital board config
    if len(virtualDigitalBoards):

      # Creates a dummy T1 physical config
      # Pretty much all physical config ignored with virtual boards,
      #   but must still be provided (default provided)
      physicalDigitalConfig = model.addT1SangomaDigitalConfig(
          model.currentConfig,
          'Virtual Digital Config',
          model.defaultT1Config,
          model.defaultAudioParams)

      # Virtual boards must have one set of interfaces using TERMINAL
      #   termination peered to interfaces using NETWORK termination
      fasConfigParams = model.defaultFasConfig

      fasConfigParams['displayName'] = 'Terminal'
      fasConfigParams['termination'] = 'TERMINAL'

      fasTerminalConfig = model.addFasConfig(model.currentConfig,
          **fasConfigParams)

      fasConfigParams['displayName'] = 'Network'
      fasConfigParams['termination'] = 'NETWORK'
      fasNetworkConfig = model.addFasConfig(model.currentConfig,
          **fasConfigParams)

      BChannelRange = model.BChannelRange.factory(firstChannel=1,
          lastChannel=23, channelGroupID='default')

      for board in virtualDigitalBoards:
        # Physical config applies to all interfaces of virtual digital board
        model.applyConfigToBoardInterfaces(physicalDigitalConfig, board)

        half = len(board.get_interface()) /2

        # Apply TERMINAL config to first half of interfaces
        for interface in board.get_interface()[:half]:
          model.applyConfigToInterface(fasTerminalConfig, board,
              interface, [BChannelRange])

        # Apply NETWORK config to second half of interfaces
        for interface in board.get_interface()[half:]:
          model.applyConfigToInterface(fasNetworkConfig, board,
              interface, [BChannelRange])

    
    # Done with boards configuration
    # Write current config to file using default file location
    model.writeCurrentConfigToFile()

    
    # Using netborder-properties-editor tool, regroup gw.properties,
    # gw_internal.properties and install.properties into gw.properties
    #
    # !+! jpboily: kept this call for development mode only. The consolidation
    # for installed version of the gateway is done in the nsi file.
    self.consolidatePropertiesFile()
    

    # Load the gw.properties file into a ParameterTrie
    paramTrie = ParameterTrie()
    if paramTrie.load(self.vInputFilePropertiesFiles, self.paramDB) != True :
      log.error("failed to load properties file: " +
                self.strGwPropFile );
      return simplejson.dumps({'success':False,'executeScript':False})

    # Applying default values
    log.debug( "adding properties with default values" );
    self.paramDB.addDefaultsToConfig( paramTrie, False );

    # Update routing rules and gw.properties.
    self.replaceTags( "data/templates/T_routing-rules.xml",
                      "config/routing-rules.xml",
                      "XXX_DEFAULT_SIP_URL_XXX",
                      "%s:%s" % (physicalConfig['sipOutAddress'], physicalConfig['sipOutPort']))

    self.replaceTags( "config/routing-rules.xml",
                      "config/routing-rules.xml",
                      "XXX_CANONICAL_MASK_XXX",
                      physicalConfig['outbound_mask'])

    self.replaceTags( "config/routing-rules.xml",
                      "config/routing-rules.xml",
                      "XXX_OUTBOUND_PREFIX_XXX",
                      physicalConfig['outbound_prefix'])

    self.replaceTags( "config/routing-rules.xml",
                      "config/routing-rules.xml",
                      "XXX_COUNTRY_CODE_XXX",
                      physicalConfig['country_code'])

    self.replaceTags( "config/routing-rules.xml",
                      "config/routing-rules.xml",
                      "XXX_DEFAULT_SIP_TRANSPORT_XXX",
                      physicalConfig['outTransportProtocol'])

    strIPAddress = \
      "INADDR_ANY:%s/udp,INADDR_ANY:%s/tcp" % (physicalConfig['udpPort'],physicalConfig['tcpPort'])

    if physicalConfig['outTransportProtocol'] == "tls" and 'enableTls' in physicalConfig:
      strIPAddress = "INADDR_ANY:%s/tls" % physicalConfig['tlsPort']
    if 'enableUdp' in physicalConfig and not 'enableTcp' in physicalConfig:
      strIPAddress = "INADDR_ANY:%s/udp" % physicalConfig['udpPort']
    elif 'enableTcp' in physicalConfig and not 'enableUdp' in physicalConfig:
      strIPAddress = "INADDR_ANY:%s/tcp" % physicalConfig['tcpPort']

    # Change the sip.userAgent.IPAddress value
    self.setParameterValue( "netborder.sip.userAgent.IPAddress", strIPAddress, paramTrie)

    # Change the primary IP address value
    self.setParameterValue( "netborder.net.primaryIPAddress", physicalConfig['primaryIPAddress'], paramTrie )

    if bHasFxsInterfaces:
      # Generate analog phone numbers string for the default analog routing rule
      strPhoneNumbers = ""
      for phoneNumber in phoneNumbersArray:
        strPhoneNumber = "%s" % phoneNumber
        if strPhoneNumbers != "":
          strPhoneNumbers =  strPhoneNumbers + "|"
        strPhoneNumbers = strPhoneNumbers + strPhoneNumber

      self.replaceTags( "config/routing-rules.xml",
                        "config/routing-rules.xml",
                        "XXX_FXS_PHONE_NUMBERS_XXX",
                        strPhoneNumbers)

    srtpMode = "as-needed"
    if 'srtpMode' in physicalConfig:
      if physicalConfig['srtpMode'] == 'Require Encryption':
        srtpMode = "always"
      elif physicalConfig['srtpMode'] == 'Do not support encryption':
        srtpMode = "never"
    else:
      log.error("No srtpMode in physicalConfig")

    # Change srtp.mode value
    self.setParameterValue( "netborder.media.srtp.mode", srtpMode, paramTrie )

    status.changeGWStartupType('autoStartGW' in physicalConfig)

    status.changeGWFailureStartupType('failureAutoStartGW' in physicalConfig)

    # Failover handling
    if ('failoverSipOutAddress' in physicalConfig and 'failoverSipOutPort' in physicalConfig):
      if (physicalConfig['failoverSipOutAddress'] != '' and physicalConfig['failoverSipOutPort'] != ''):
        self.replaceTags( "config/routing-rules.xml",
                          "config/routing-rules.xml",
                          "XXX_FAILOVER_SIP_URL_XXX",
                          "%s:%s" % (physicalConfig['failoverSipOutAddress'],
                          physicalConfig['failoverSipOutPort']))

        if 'failoverSipTimeout' in physicalConfig:
          # Notice the 000 added on purpose to convert Seconds to Ms
          self.replaceTags( "config/routing-rules.xml",
                            "config/routing-rules.xml",
                            "<!-- <param name=\"netborder.gw.ringTimeoutMs\" expr=\"XXX_DEFAULT_RING_TIMEOUT_XXX\"/> -->",
                            "<param name=\"netborder.gw.ringTimeoutMs\" expr=\"%s000\"/>" % physicalConfig['failoverSipTimeout'])
        else:
          log.error("No failoverSipTimeout in physicalConfig")

        # Change the gw.maxRoutingRulesMatches value
        self.setParameterValue( "netborder.gw.maxRoutingRulesMatches", "2", paramTrie )
    else:
      self.replaceTags("config/routing-rules.xml",
                       "config/routing-rules.xml",
                       "      <!-- The following parameter is only used in the case of failover -->\n" +
                       "      <!-- <param name=\"netborder.gw.ringTimeoutMs\" expr=\"XXX_DEFAULT_RING_TIMEOUT_XXX\"/> -->\n",
                       "");

      self.replaceTags("config/routing-rules.xml",
                       "config/routing-rules.xml",
                       "\n" +
                       "  <!-- failover ROUTING RULES -->\n" +
                       "  <!-- SIP OUT TO sip:dnis@XXX_FAILOVER_SIP_URL_XXX -->\n" +
                       "  <rule name=\"backup_sip_out\" outbound_interface=\"sip\" qvalue=\"0.0001\">\n" +
                       "    <condition param=\"transfer\" expr=\"false\"/>\n" +
                       "    <condition param=\"pstn.in.channelName\" expr=\".*\"/>\n" +
                       "    <condition param=\"pstn.in.dnis\" expr=\"(.*)\"/>\n" +
                       "    <condition param=\"pstn.in.ani\" expr=\"(.*)\"/>\n" +
                       "    <condition param=\"pstn.in.callerName\" expr=\"(.*)\"/>\n" +
                       "    <condition param=\"pstn.in.ani.presentationIndicator\" expr=\"(allowed|^$)\"/>\n" +
                       "    <out_leg name=\"\" media_type=\"sendrecv\">\n" +
                       "      <!-- To modify the target SIP destination, just change the value below -->\n" +
                       "      <param name=\"sip.out.requestUri\" expr=\"sip:%0@XXX_FAILOVER_SIP_URL_XXX;user=phone\"/>\n" +
                       "      <param name=\"sip.out.from.displayName\" expr=\"&quot;%2&quot;\"/>\n" +
                       "      <param name=\"sip.out.from.uri\" expr=\"sip:%1@GW_HOST_IP:GW_SIP_PORT\"/>\n" +
                       "      <param name=\"sip.out.to.displayName\" expr=\"&quot;%0&quot;\"/>\n" +
                       "      <param name=\"sip.out.to.uri\" expr=\"sip:%0@XXX_FAILOVER_SIP_URL_XXX\"/>\n" +
                       "      <param name=\"sip.out.transport\" expr=\"udp\"/>\n" +
                       "    </out_leg>\n" +
                       "  </rule>\n" +
                       "\n" +
                       "  <!-- SIP OUT TO sip:dnis@XXX_FAILOVER_SIP_URL_XXX -->\n" +
                       "  <rule name=\"backup_sip_out_restricted\" outbound_interface=\"sip\"\n" +
                       "        qvalue=\"0.0001\">\n" +
                       "    <condition param=\"transfer\" expr=\"false\"/>\n"+
                       "    <condition param=\"pstn.in.channelName\" expr=\".*\"/>\n" +
                       "    <condition param=\"pstn.in.dnis\" expr=\"(.*)\"/>\n" +
                       "    <condition param=\"pstn.in.ani\" expr=\"(.*)\"/>\n" +
                       "    <condition param=\"pstn.in.callerName\" expr=\"(.*)\"/>\n" +
                       "    <condition param=\"pstn.in.ani.presentationIndicator\" expr=\"restricted\"/>\n" +
                       "    <out_leg name=\"\" media_type=\"sendrecv\">\n" +
                       "      <!-- To modify the target SIP destination, just change the value below -->\n" +
                       "      <param name=\"sip.out.requestUri\" expr=\"sip:%0@XXX_FAILOVER_SIP_URL_XXX;user=phone\"/>\n" +
                       "      <param name=\"sip.out.from.displayName\" expr=\"&quot;anonymous&quot;\"/>\n" +
                       "      <param name=\"sip.out.from.uri\"\n" +
                       "             expr=\"sip:anonymous@GW_HOST_IP:GW_SIP_PORT\"/>\n" +
                       "      <param name=\"sip.out.to.displayName\" expr=\"&quot;%0&quot;\"/>\n" +
                       "      <param name=\"sip.out.to.uri\" expr=\"sip:%0@XXX_FAILOVER_SIP_URL_XXX\"/>\n" +
                       "      <param name=\"sip.out.transport\" expr=\"udp\"/>\n" +
                       "    </out_leg>\n" +
                       "  </rule>\n" +
                       "\n",
                       "");

    # TODO - replace the running on HP check with Using OCS checkbox value
    if globalConfig.getParameter("netborder.license.runningOnHpEsp") :
      log.debug("running on HP")
      self.setParameterValue( "netborder.net.HostNameServer.DNSServerIPAddrList", "use-host-dns-list", paramTrie )
      self.setParameterValue( "netborder.sip.in.send183wSDPBeforeMediaAvailable", "true", paramTrie )
      self.setParameterValue( "netborder.media.rtp.lockOnSDP", "true", paramTrie )
      self.setParameterValue( "netborder..media.vad.silenceDecisionDebounceMs", "850", paramTrie )
      self.setParameterValue( "netborder.media.vad.sensitivity", "2", paramTrie )
      self.setParameterValue( "netborder.gw.failoverMode", "stopFailoverOnSipMessage", paramTrie )
    else :
      log.debug("not running on HP")

    # Save the properties value in gw.properties
    if not paramTrie.saveAsPropertiesFile( self.strGwPropFile, self.paramDB,
                                           True ):
      log.error("failed to save output properties to file: " +
                self.strGwPropFile)
      return simplejson.dumps({'success':False,'executeScript':False})

    # See if we need to execute a post quick setup script
    log.debug( "Post Quick Setup Cmd value: %s" % self.postQuickSetupCmd )
    if self.postQuickSetupCmd:
      executeScript = True
    else:
      executeScript = False

    # Invoked by Ext JS Form "submit": JSON response expected
    return simplejson.dumps({'success':True,'hasFxo':bHasFxoInterfaces,'executeScript':executeScript})

  #############################################################################
  # This method will use netborder-properties-editor tool to consolidate the
  # properties files (gw.properties, gw_internal.properties and
  # install.properties) into gw.properties file only. The consolidation is done
  # once, then we remove the gw_internal.properties and install.properties
  # files. When going through this method, at the beginning, we check that the
  # gw_internal.properties file exists. If not, we return immediately. After
  # the consolidation, we update the Path to executable for the gateway service
  # since at installation, we needed to put the --gl install.properties switch.
  #############################################################################
  def consolidatePropertiesFile(self):

    gw_internal_path = \
      os.path.join(self.root_path, "config/gw_internal.properties")

    if not os.path.isfile(gw_internal_path):
      log.debug("could not find gw_internal.properties. assuming consolidation is already done.")
      return

    # To consolidate properties files, the netborder-properties-editor tool
    # takes the following command:
    #   netborder-properties-editor --param-db [paramDefs_file] prop_file1 prop_file2 [...]
    # the -c "add-defaults" option will force the tool to add the missing
    # parameters with their default values in the output file. Since the output
    # we want contains some pre-defined values for some parameters, we must
    # give in entry as well as in output.
    if os.name == 'nt':
      propEditor = "netborder-properties-editor"
    else:
      propEditor = self.root_path + "/bin/netborder-properties-editor"

    args = [
      propEditor,
      "-c",
      "\"add-defaults\"",
      "--param-db",
      self.root_path + "/config/netborder-gateway-paramDefs.xml",
      "-o",
      self.root_path + "/config/gw.properties",
      self.root_path + "/config/gw.properties",
      self.root_path + "/config/install.properties",
      self.root_path + "/config/gw_internal.properties"
    ]

    p = subprocess.Popen(
      args=args,
      stdin=subprocess.PIPE,
      stdout=subprocess.PIPE,
      stderr=subprocess.PIPE
    )

    strOutput = ""
    strOutput = strOutput + p.communicate()[0]
    log.debug("Post Script result: %s" % strOutput)

    # Remove gw_internal.properties
    file = self.root_path + "/config/gw_internal.properties"
    os.remove(file)

    # Remove install.properties
    file = self.root_path + "/config/install.properties"
    os.remove(file)

    # Update gw service path to include only the gw.properties for global
    # config file
    serviceName = \
      globalConfig.getParameter("netborder.oamWebServer.gwServiceName")

    gwInCLI = globalConfig.getParameter("netborder.oamWebServer.isGatewayInCLI")
    log.debug("Is gateway in CLI: %s" % gwInCLI)

    if not gwInCLI:

      if os.name == 'nt':
        binPath = "\"" + self.root_path + \
            "/bin/netborder-gateway.exe\" --setcwd \"" + \
            self.root_path + "/bin\" --global-config \"" + self.root_path + \
            "/config/gw.properties\" --service --svc-name \"" + \
            serviceName + "\""

        log.debug("new bin path: %s" % binPath)
        args = [
          "sc",
          "config",
          serviceName,
          "binPath=",
          binPath
          ]

      else:

        s1 = "NETBORDER_GATEWAY_OPTIONS=.*"
        s2 = "NETBORDER_GATEWAY_OPTIONS=\"--service --setcwd" + \
            self.root_path + "/bin --global-config " + \
             self.root_path + "/config/gw.properties"

        args = [
          "sed",
          "-si",
          "s/" + s1 + "/" + s2 + "/g"
          "/etc/sysconfig/netborder-gateway.conf"
          ]

      p = subprocess.Popen(
        args=args,
        stdin=subprocess.PIPE,
        stdout=subprocess.PIPE,
        stderr=subprocess.PIPE
      )
      log.info("updated gateway service command line: %s" % p.communicate()[0])

  #############################################################################
  # This method sets a new value for a parameter in the ParameterTrie.
  # @params:
  #   - in_paramName: the parameter name
  #   - in_paramValue: the new parameter value
  #   - inout_paramTrie: the ParameterTrie in which to set the new value
  # @returns: True if everything worked; False if there was an error
  #############################################################################
  def setParameterValue( self, in_paramName, in_paramValue, inout_paramTrie):
    log.debug("setting parameter [%s] value to [%s]" % (in_paramName,in_paramValue))
    param = Parameter()

    in_paramValue = in_paramValue.encode("utf-8")

    result = param.initFromText( in_paramName,
                                 in_paramValue,
                                 self.paramDB )
    if result != SUCCESS_INITRESULT:
      if result == PARAM_NOT_IN_DB_INITRESULT:
        log.error( "failed to set parameter " + in_paramName + " to value " + \
                   in_paramValue + " because the specified parameter does " \
                   "not exist in the parameter DB")
      elif result == PARAM_WRONG_TYPE_INITRESULT :
        log.error( "failed to set parameter " + in_paramName + " to value " + \
                   in_paramValue + " because the specified parameter value " \
                   "type is wrong")
      else:
        log.error( "failed to set parameter " + in_paramName + " to value " + \
                   in_paramValue + " because the parameter value is missing")

      return False

    if inout_paramTrie.setParameter( in_paramName, in_paramValue ) == False:
      log.error( "failed to set parameter " + in_paramName )
      return False

    return True

  def replaceTags(self,
                  in_strTemplateFileName,
                  in_strFilename,
                  in_strPattern,
                  in_strValue):
    """Finds all the occurence of the in_strPattern we need to replace in
       the file and change them for the corresponding value in_strValue"""

    log.info("Replacing tag [%s] with tag [%s]" % (in_strPattern,in_strValue))
    # Open file
    templates_path = \
      globalConfig.getParameter("netborder.Installation.Directory")

    if templates_path is None:
      log.error("netborder.Installation.Directory parameter not set. Cannot "\
                 "change value in file %s", in_strFilename)
      return

    strTemplateFullFileName = \
      os.path.join(templates_path, in_strTemplateFileName)

    strFullFileName = os.path.join(templates_path, in_strFilename)

    if not os.path.isfile(strTemplateFullFileName):
      log.error("Cannot find file %s", strTemplateFullFileName)
      return

    # Open template file and get its content.
    try:
      f = open(strTemplateFullFileName,'r')
      # Read all content
      strFileContent = f.read()
      f.close()
    except IOError:
      log.error("Cannot open file %s", strTemplateFullFileName)
      return

    if strFileContent == "":
      log.error("Error file %s is empty",strTemplateFullFileName)

    # Replace string in file file.
    strFileContent=strFileContent.replace(in_strPattern, in_strValue)

    # Overwrite file with new content
    try:
       f = open(strFullFileName,'w')
       f.write(strFileContent)
       f.close()
    except IOError:
       log.error("Cannot open file %s", strFullFileName)
       return

  #----------------------------------------------------------------------------
  # Adds Fxo and Fxs call control config and apply them to the corresponding
  # interfaces.
  # @param model: the pstnconfig model representation
  # @param board: instance of the analog board we are adding
  # @param phoneNumbers: an array of all the fxs phone numbers used to append
  #                      the phone numbers of this board.
  #----------------------------------------------------------------------------
  def addAnalogConfigs( self,
                        model,
                        board,
                        phoneNumbers):
    hasFxoInterfaces = model.boardHasFxoInterfaces(board)
    hasFxsInterfaces = model.boardHasFxsInterfaces(board)

    config = model.currentConfig

    if hasFxoInterfaces:
      configParams = {}
      configParams = \
      {
        'displayName':'Config%s-FXO' % model.getNextAnalogConfID(config)
      }
      log.debug("Adding analog fxo config [%s]" % configParams['displayName'])
      analogFxoConfig = \
        model.addAnalogFxoConfig(config,configParams)
      for interface in board.get_interface():
        if interface.get_type() == "FXO" :
          model.applyConfigToInterface(
            analogFxoConfig,
            board,
            interface,
            channelGroupID="default"
          )

    if hasFxsInterfaces:
      configParams = {}
      # Get default parameters for an Fxs config
      configParams = model.defaultFxsParams
      # Set the display name of the Fxs config
      configParams['displayName'] = \
        'Config%s-FXS' % model.getNextAnalogConfID(config)

      log.debug("Adding analog fxs config [%s]" % configParams['displayName'])
      # Create the Fxs config
      analogFxsConfig = \
        model.addAnalogFxsConfig(config,configParams)
      # For each FXS interface of the board,
      for interface in board.get_interface():
        if interface.get_type() == "FXS":
          phoneNumbers.append(interface.get_phoneNumber())
          # Apply the create FXS config to those interface
          model.applyConfigToInterface(
            analogFxsConfig,
            board,
            interface,
            channelGroupID=interface.get_phoneNumber()
          )

    return True

  # Returns a dictionary of all the IP address available on the system
  @expose()
  def getIPAddress(self, js=''):
    data = []
    if sys.platform == 'linux':
      import fcntl
      SIOCGIFCONF = 0x8912  #define SIOCGIFCONF
      BYTES = 4096          #Simply define the byte size
      SIOCGIFADDR = 0x8915  #define SIOCGIFADDR
      # get the if list
      # create the socket object to get the interface list
      sck = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
      # prepare the struct variable
      names = array.array('B', '\0' * BYTES)

      # the trick is to get the list from ioctl
      bytelen = struct.unpack('iL', fcntl.ioctl(sck.fileno(), SIOCGIFCONF, struct.pack('iL', BYTES, names.buffer_info()[0])))[0]

      # convert it to string
      namestr = names.tostring()

      # the interfaces as array
      ifaces = [namestr[i:i+32].split('\0', 1)[0] for i in range(0, bytelen, 32)]
      # well, what to do? print it out maybe... 
      for iface in ifaces:
        if iface != 'lo':
          ip = socket.inet_ntoa(fcntl.ioctl(sck.fileno(),SIOCGIFADDR,struct.pack('256s', iface[:15]))[20:24])
          log.debug("Interace : %s, IP Address : %s" % (iface,ip))
          data.append({"ip":ip})

      data.append({"ip":'127.0.0.1'})

    else:
      # Get the extended list of all IP addifaceress for this system
      # Returns a tuple (hostname, aliaslist, ipaddrlist)
      ipList = socket.gethostbyname_ex(socket.gethostname())

      # The IP address list is always located at index 2
      ipList = ipList[2]
      log.debug("IP Address list: %s" % ipList)

      for ip in ipList:
        data.append({"ip":ip})

      data.append({"ip":'127.0.0.1'})

    return(str(extjs.FormActionLoadResponse(success=True, data=data)))

  # Executes the command specified in netborder.oamWebServer.postQuickSetupCmd
  # and returns the stdout and stderr values combined in a string.
  @expose()
  def executePostQuickSetupCmd(self, js=''):
    # Add the command at the first line of the resulting output
    strOutput = ""

    # Split the command using shlex library which basically splits a command
    # like Unix's shell (http://docs.python.org/library/shlex.html)
    args = shlex.split(self.postQuickSetupCmd)

    # Split the command string according to the spaces between the arguments
    #args = self.postQuickSetupCmd.split()
    log.info("Post Script command: %s" % args)

    # Open a process that will execute the arguments specified in args
    # stdout=subprocess.PIPE tells to open a standard stream pipe
    # stderr=subprocess.STDOUT tells to append stderr to stdout
    # p is the resulting pipe
    p = \
    subprocess.Popen(
      args=args,
      stdin=subprocess.PIPE,
      stdout=subprocess.PIPE,
      stderr=subprocess.STDOUT
    )

    # p.communicate() returns a tuple of the stdout and stderr (stdout,stderr)
    # Since we merged stdout and stderr, we only need stdout's value
    strOutput = strOutput + p.communicate()[0]
    log.debug("Post Script result: %s" % strOutput)
    return {'scriptResult':strOutput}

@expose()
def default(self, *args, **kw):
  # redirect to front page
  redirect("/")

